/*!
 * @vueblocks/vue-use-core v0.2.4
 * (c) 2021 xiaoluoboding
 * @license MIT
 */
(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('vue-demi'), require('color2k'), require('lodash.debounce'), require('@juggle/resize-observer'), require('screenfull'), require('lodash.throttle'), require('keymaster')) :
  typeof define === 'function' && define.amd ? define(['exports', 'vue-demi', 'color2k', 'lodash.debounce', '@juggle/resize-observer', 'screenfull', 'lodash.throttle', 'keymaster'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.VueUseUtilities = {}, global.VueDemi, global.color2k, global.lodash.debounce, global.ResizeObserver, global.screenfull, global.lodash.throttle, global.keymaster));
}(this, (function (exports, vueDemi, color2k, debounce, resizeObserver, screenfull, throttle, keymaster) { 'use strict';

  function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

  var color2k__default = /*#__PURE__*/_interopDefaultLegacy(color2k);
  var debounce__default = /*#__PURE__*/_interopDefaultLegacy(debounce);
  var screenfull__default = /*#__PURE__*/_interopDefaultLegacy(screenfull);
  var throttle__default = /*#__PURE__*/_interopDefaultLegacy(throttle);
  var keymaster__default = /*#__PURE__*/_interopDefaultLegacy(keymaster);

  const cssVariableRegex = /--[\S]*/g;
  const useCssVars = (el, variable, defaultValue) => {
      if (!variable || !variable.match(cssVariableRegex)) {
          console.warn(`[useCssVars] css variable is not valid: ${variable}`);
      }
      const propertyValue = vueDemi.ref('');
      const target = vueDemi.ref(el || window.document.documentElement);
      vueDemi.watch(target, (el) => {
          if (window && el) {
              const propValue = window.getComputedStyle(el).getPropertyValue(variable) || defaultValue || '';
              propertyValue.value = propValue;
          }
      }, { immediate: true });
      vueDemi.watch(propertyValue, (newVal) => {
          if (target.value && target.value.style) {
              target.value.style.setProperty(variable, newVal);
          }
      });
      return propertyValue;
  };

  const isBoolean = (val) => typeof val === 'boolean';
  const isClient = ![typeof window, typeof document].includes('undefined');
  const isElement = (val) => typeof val === 'object' && !!val.tagName;
  const isUndefined = (val) => val === undefined;

  const NOOP = () => { };

  /**
   * enhanced `color2k` with new utilities like `tint` 、 `shade` 、 `invert`.
   */
  const usePalette = {
      /**
       * Tints a color by mixing it with white.
       * @param color
       * @param amount given as a decimal between 0 and 1
       */
      tint: (color, amount) => color2k__default['default'].mix('#FFF', color, amount),
      /**
       * Shades a color by mixing it with black.
       * @param color
       * @param amount given as a decimal between 0 and 1
       */
      shade: (color, amount) => color2k__default['default'].mix('#000', color, amount),
      /**
       * Inverts the red, green and blue values of a color.
       * @param color
       */
      invert: (color) => {
          if (color === 'transparent')
              return color;
          const [r, g, b, a] = color2k__default['default'].parseToRgba(color);
          return color2k__default['default'].rgba(255 - r, 255 - g, 255 - b, a);
      },
      ...color2k__default['default']
  };
  /**
   * Reactive color parsing and manipulation built on top of color2k.
   * @param color
   * @param amount
   */
  const useColor = (color, amount) => {
      const { tint, shade, invert } = usePalette;
      const colorState = {
          hexColor: vueDemi.computed(() => ''),
          hslaColor: vueDemi.computed(() => ''),
          rgbaColor: vueDemi.computed(() => ''),
          readableColor: vueDemi.computed(() => ''),
          invertColor: vueDemi.computed(() => ''),
          luminance: vueDemi.computed(() => 0),
          lightenColor: vueDemi.computed(() => ''),
          darkenColor: vueDemi.computed(() => ''),
          tintColor: vueDemi.computed(() => ''),
          shadeColor: vueDemi.computed(() => ''),
          desaturateColor: vueDemi.computed(() => ''),
          saturateColor: vueDemi.computed(() => ''),
          opacifyColor: vueDemi.computed(() => ''),
          transparentizeColor: vueDemi.computed(() => '')
      };
      // accpet color argument
      if (vueDemi.isRef(color) && vueDemi.unref(color)) {
          colorState.hexColor = vueDemi.computed(() => color2k__default['default'].toHex(vueDemi.unref(color)));
          colorState.hslaColor = vueDemi.computed(() => color2k__default['default'].toHsla(vueDemi.unref(color)));
          colorState.rgbaColor = vueDemi.computed(() => color2k__default['default'].toRgba(vueDemi.unref(color)));
          colorState.readableColor = vueDemi.computed(() => color2k__default['default'].readableColor(vueDemi.unref(color)));
          colorState.invertColor = vueDemi.computed(() => invert(vueDemi.unref(color)));
          colorState.luminance = vueDemi.computed(() => color2k__default['default'].getLuminance(vueDemi.unref(color)));
      }
      // accept amount argument
      if (vueDemi.isRef(amount) && !isUndefined(vueDemi.unref(amount))) {
          colorState.lightenColor = vueDemi.computed(() => color2k__default['default'].lighten(vueDemi.unref(color), vueDemi.unref(amount)));
          colorState.darkenColor = vueDemi.computed(() => color2k__default['default'].darken(vueDemi.unref(color), vueDemi.unref(amount)));
          colorState.tintColor = vueDemi.computed(() => tint(vueDemi.unref(color), vueDemi.unref(amount)));
          colorState.shadeColor = vueDemi.computed(() => shade(vueDemi.unref(color), vueDemi.unref(amount)));
          colorState.desaturateColor = vueDemi.computed(() => color2k__default['default'].desaturate(vueDemi.unref(color), vueDemi.unref(amount)));
          colorState.saturateColor = vueDemi.computed(() => color2k__default['default'].saturate(vueDemi.unref(color), vueDemi.unref(amount)));
          colorState.opacifyColor = vueDemi.computed(() => color2k__default['default'].opacify(vueDemi.unref(color), vueDemi.unref(amount)));
          colorState.transparentizeColor = vueDemi.computed(() => color2k__default['default'].transparentize(vueDemi.unref(color), vueDemi.unref(amount)));
      }
      return colorState;
  };

  const useDebouncedRef = (value, delay = 200) => {
      return vueDemi.customRef((track, trigger) => {
          return {
              get() {
                  track();
                  return value;
              },
              set: debounce__default['default']((newValue) => {
                  value = newValue;
                  trigger();
              }, delay)
          };
      });
  };

  /**
   * It immediately detects when an element resizes and provides accurate sizing information back to the handler.
   * @param target
   * @param callback
   * @param options
   */
  const useResizeObserver = (target, callback, options) => {
      let ro;
      const targetRef = vueDemi.ref(target);
      const isSupported = window && 'ResizeObserver' in window;
      const stopWatch = vueDemi.watch(targetRef, (newVal) => {
          if (isSupported && window && newVal) {
              ro = new resizeObserver.ResizeObserver(callback);
              ro.observe(newVal, options);
          }
      }, { immediate: true });
      const stop = () => {
          ro && ro.disconnect();
          ro = undefined;
          stopWatch();
      };
      return {
          stop
      };
  };

  const useElementSize = (target, options) => {
      const width = vueDemi.ref(isElement(target) ? target.clientWidth : 0);
      const height = vueDemi.ref(isElement(target) ? target.clientHeight : 0);
      useResizeObserver(target, ([entry]) => {
          width.value = entry.contentRect.width;
          height.value = entry.contentRect.height;
      }, options);
      return { width, height };
  };

  function useEventListener(target, event, listener, options) {
      let remove = NOOP;
      if (target) {
          const element = vueDemi.ref(target);
          const removeEventListener = (e) => e.removeEventListener(event, listener);
          const addEventListener = (e) => e.addEventListener(event, listener, options);
          const removeWatch = vueDemi.watch(element, (n, _, cleanUp) => {
              if (n) {
                  addEventListener(n);
                  cleanUp(() => removeEventListener(n));
              }
          }, { immediate: true });
          remove = () => {
              removeEventListener(element.value);
              removeWatch();
          };
      }
      return remove;
  }
  // export { useEventListener }

  /**
   * Call onMounted() if it's inside a component lifecycle, if not, run just call the function
   * @param fn
   * @param sync if set to false, it will run in the nextTick() of Vue
   */
  function tryOnMounted(fn, sync = true) {
      if (vueDemi.getCurrentInstance()) {
          vueDemi.onMounted(fn);
      }
      else if (sync) {
          fn();
      }
      else {
          vueDemi.nextTick(fn);
      }
  }
  /**
   * Call onUnmounted() if it's inside a component lifecycle, if not, do nothing
   * @param fn
   */
  function tryOnUnmounted(fn) {
      if (vueDemi.getCurrentInstance())
          vueDemi.onUnmounted(fn);
  }

  /**
   * Reactive Fullscreen API build on top of screenfull.js
   * @param {HTMLElement} target
   * @return {boolean} isFullscreen
   * @return {function} toggleFullscreen
   */
  const useFullscreen = (target) => {
      const targetRef = vueDemi.ref(target || document.querySelector('html'));
      const isFullscreen = vueDemi.ref(false);
      const toggleFullscreen = () => {
          screenfull__default['default'].isEnabled && screenfull__default['default'].toggle(targetRef.value);
      };
      tryOnMounted(() => {
          if (screenfull__default['default'].isEnabled) {
              screenfull__default['default'].on('change', () => {
                  isFullscreen.value = screenfull__default['default'].isFullscreen;
              });
          }
      });
      return {
          isFullscreen,
          toggleFullscreen
      };
  };

  const useInstance = () => {
      const instance = vueDemi.getCurrentInstance();
      if (!instance) {
          console.error('`getCurrentInstance` only works during setup or Lifecycle Hooks');
          return null;
      }
      return vueDemi.isVue3 ? instance.ctx : instance.proxy;
  };

  /**
   * Register `MouseMove` event on mounted, and remove event automatically on unmounted.
   */
  const useMouse = () => {
      let remove = NOOP;
      const x = vueDemi.ref(0);
      const y = vueDemi.ref(0);
      // const touchHandler = (event: TouchEvent) => {
      //   if (event.touches.length > 0) {
      //     x.value = event.touches[0].clientX
      //     y.value = event.touches[0].clientY
      //   }
      // }
      if (window) {
          const mouseHandler = (event) => {
              x.value = event.pageX;
              y.value = event.pageY;
          };
          remove = useEventListener(window, 'mousemove', mouseHandler, { passive: true });
      }
      return {
          x,
          y,
          remove
      };
  };

  function useReactiveState(getter) {
      const state = vueDemi.reactive(getter());
      const update = () => {
          Object.assign(state, getter());
      };
      return [state, update];
  }

  /**
   * Reactive Resize API, When window resize, do someting.
   * @param {function} fn
   * @param {object} options
   * @return {function} dispatchResize
   */
  const useResizeEvent = (fn, options = { useCapture: false, useThrottle: null, delay: 500 }) => {
      const { useCapture = false, useThrottle = null, delay = 500 } = options;
      const dispatchResize = () => {
          const e = document.createEvent('Event');
          e.initEvent('resize', true, true);
          window.dispatchEvent(e);
      };
      const waitFn = isBoolean(useThrottle)
          ? useThrottle
              ? throttle__default['default'](fn, delay)
              : debounce__default['default'](fn, delay)
          : fn;
      tryOnMounted(() => {
          window.addEventListener('resize', waitFn, useCapture);
      });
      tryOnUnmounted(() => {
          window.removeEventListener('resize', waitFn, useCapture);
      });
      return {
          dispatchResize
      };
  };

  const bindKeyHandler = (fn) => {
      return (...args) => {
          fn(...args);
          return false;
      };
  };
  /**
   * Given shortcut a new bind function
   */
  const shortcut = {
      bind: (seed, func) => keymaster__default['default'](seed, bindKeyHandler(func)),
      ...keymaster__default['default']
  };
  /**
   * Bind shortcut onMounted & unbind shortcut automatically onUnmounted in setup hooks
   * or
   * Unbind shortcut before bind shortcut without setup hooks
   * @param {Object} keymap
   * @param {Object} vm
   * @return {function} toggle
   */
  const useShortcut = (keymap, vm) => {
      let sync = true;
      if (!vm) {
          vm = vueDemi.getCurrentInstance();
      }
      else {
          sync = false;
          unbindKeys();
      }
      function bindKeys() {
          Object.keys(keymap).forEach(key => {
              shortcut.bind(key, keymap[key]);
          });
      }
      function unbindKeys() {
          Object.keys(keymap).forEach(key => {
              shortcut.unbind(key);
          });
      }
      tryOnMounted(() => bindKeys(), sync);
      tryOnUnmounted(unbindKeys);
      return shortcut;
  };

  /**
   * Reactive window size
   * @param {object} options
   * @return {Ref<number>} WindowSize
   */
  const useWindowSize = (options) => {
      const width = vueDemi.ref(isClient ? window.innerWidth : 0);
      const height = vueDemi.ref(isClient ? window.innerHeight : 0);
      const onResize = () => {
          width.value = window.innerWidth;
          height.value = window.innerHeight;
      };
      const { dispatchResize } = useResizeEvent(onResize, options);
      vueDemi.onMounted(dispatchResize);
      return { width, height };
  };

  exports.tryOnMounted = tryOnMounted;
  exports.tryOnUnmounted = tryOnUnmounted;
  exports.useColor = useColor;
  exports.useCssVars = useCssVars;
  exports.useDebouncedRef = useDebouncedRef;
  exports.useElementSize = useElementSize;
  exports.useEventListener = useEventListener;
  exports.useFullscreen = useFullscreen;
  exports.useInstance = useInstance;
  exports.useMouse = useMouse;
  exports.usePalette = usePalette;
  exports.useReactiveState = useReactiveState;
  exports.useResizeEvent = useResizeEvent;
  exports.useResizeObserver = useResizeObserver;
  exports.useShortcut = useShortcut;
  exports.useWindowSize = useWindowSize;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
